/***********************************/
/****** NRF24L01 Declaration *******/
/***********************************/

#include <stdint.h>

/***** Pin Definitions *************/
/***** Depends on the AVR **********/

#define NRF_CE 			PB4
#define NRF_CE_PORT 	PORTB
#define NRF_CE_DDR 		DDRB

#define NRF_IRQ			PINB2
#define NRF_IRQ_PORT	PORTB
#define NRF_IRQ_DDR		DDRB
#define NRF_IRQ_PIN		PINB

/***** Interrupt Setup *************/

#define NRF_INT_SENS		EICRA = /*(1<<ISC01) |*/ (0<<ISC00);
#define NRF_INT_EN			EIMSK = (1<<INT0);
#define NRF_INT_DIS			EIMSK = ~(1<<INT0);
#define NRF_INT_FLAG_CL		EIFR = (1<<INTF0);

/***** Define short commands *******/

#define NRF_CE_Lo NRF_CE_PORT &= ~(1<<NRF_CE);
#define NRF_CE_Hi NRF_CE_PORT |= (1<<NRF_CE);

/***** Define Constants ************/

#define   NO_OP           0xff
#define   R_REGISTER      0x00
#define   W_REGISTER      0x20
#define   R_RX_PAYLOAD    0x61
#define   W_TX_PAYLOAD    0xA0
#define   FLUSH_TX        0xE1
#define   FLUSH_RX        0xE2
#define   REUSE_TX_PL     0xE3

#define   RX_ADDR_P0      0x0A
#define   RX_ADDR_P1      0x0B
#define   RX_ADDR_P2      0x0C
#define   RX_ADDR_P3      0x0D
#define   RX_ADDR_P4      0x0E
#define   RX_ADDR_P5      0x0F
#define   TX_ADDR         0x10
#define   RX_PW_P0        0x11
#define   RX_PW_P1        0x12
#define   RX_PW_P2        0x13
#define   RX_PW_P3        0x14
#define   RX_PW_P4        0x15
#define   RX_PW_P5        0x16
#define   FIFO_STATUS     0x17

#define   CONFIG		  0x00
#define	  EN_AA			  0x01
#define   EN_RXADDR		  0x02
#define   SETUP_AW		  0x03
#define	  SETUP_RETR	  0x04
#define   RF_CH			  0x05
#define	  RF_SETUP		  0x06	
#define   STATUS		  0x07
#define   OBSERVE_TX	  0x08
#define	  CD			  0x09
#define   DYNPD			  0x1C
#define   FEATURE		  0x1D

#define   TX_FULL         0x01
#define   RX_DR           0x40
#define   TX_DS           0x20
#define   MAX_RT          0x10

/***** Funktions *******************/

void NRF_ReuseTXBuffer(void);
void NRF_FlushTXBuffer(void);
void NRF_FlushRXBuffer(void);

struct uint256_t NRF_ReadRXPayload(void);
void NRF_WriteTXPayload(struct uint256_t);

void NRF_ChangeRXAddress(uint32_t, uint8_t);
void NRF_ChangeTXAddress(uint32_t);

uint32_t NRF_ReadRXAddress(uint8_t);
uint32_t NRF_ReadTXAddress();

uint8_t NRF_ReadRegister(uint8_t);
void NRF_WriteRegister(uint8_t, uint8_t);

void NRF24L01_Init(void);
void NRF24L01_CustomInit(uint8_t, uint8_t, uint8_t, unsigned char [4][4]);
void NRF_SetRXState(void);
void NRF_SetTXState(void);
void NRF_SendImpuls(void);




