/****************************************/
/**** NRF Send and Receive Functions ****/
/****************************************/

#include <util/delay.h>

#include "NRF24L01_Dec.h"
#include "Converter.h"

extern volatile uint8_t RFChannel;
extern volatile unsigned char GateAddress[4];

void SendBack(struct uint256_t Payload, uint8_t Pipe)
{
	uint32_t Address = 0x00000000;

	switch(Pipe)
	{
		case 0x01 :
			Address = NRF_ReadRXAddress(RX_ADDR_P1);
			break;
		case 0x02 :
			Address = NRF_ReadRXAddress(RX_ADDR_P2);
			break;
		case 0x03 :
			Address = NRF_ReadRXAddress(RX_ADDR_P3);
			break;
		case 0x04 :
			Address = NRF_ReadRXAddress(RX_ADDR_P4);
			break;
	}

	NRF_CE_Lo;
	NRF_ChangeTXAddress(Address);
	NRF_ChangeRXAddress(Address, RX_ADDR_P0);
	NRF_SetTXState();
	NRF_FlushTXBuffer();
	NRF_WriteTXPayload(Payload);

	NRF_CE_Hi;
	_delay_ms(1);
	NRF_CE_Lo;

	NRF_SendImpuls();

	while(NRF_IRQ_PIN & (1<<NRF_IRQ));

	uint8_t status_reg;
	status_reg = NRF_ReadRegister(STATUS);

	if((status_reg & RX_DR) == RX_DR)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	if((status_reg & TX_DS) == TX_DS)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	if((status_reg & MAX_RT) == MAX_RT)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	
	NRF_CE_Lo;
	NRF_SetRXState();
	NRF_CE_Hi;
}

void SendToGate(struct uint256_t Payload)
{
	Addressconvert.c[0] = GateAddress[0];
	Addressconvert.c[1] = GateAddress[1];
	Addressconvert.c[2] = GateAddress[2];
	Addressconvert.c[3] = GateAddress[3];

	NRF_CE_Lo;
	NRF_ChangeTXAddress(Addressconvert.u32i);
	NRF_ChangeRXAddress(Addressconvert.u32i, RX_ADDR_P0);
	NRF_SetTXState();
	NRF_FlushTXBuffer();
	NRF_WriteTXPayload(Payload);

	NRF_CE_Hi;
	_delay_ms(1);
	NRF_CE_Lo;

	NRF_SendImpuls();

	while(NRF_IRQ_PIN & (1<<NRF_IRQ));

	uint8_t status_reg;
	status_reg = NRF_ReadRegister(STATUS);

	if((status_reg & RX_DR) == RX_DR)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	if((status_reg & TX_DS) == TX_DS)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	if((status_reg & MAX_RT) == MAX_RT)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	NRF_CE_Lo;
	NRF_SetRXState();
	NRF_CE_Hi;
}

void SendtoLights(struct uint256_t Payload, uint32_t Address)
{
	NRF_CE_Lo;
	NRF_ChangeTXAddress(Address);
	NRF_ChangeRXAddress(Address, RX_ADDR_P0);
	NRF_SetTXState();
	NRF_FlushTXBuffer();
	NRF_WriteTXPayload(Payload);

	NRF_CE_Hi;
	_delay_ms(1);
	NRF_CE_Lo;

	NRF_SendImpuls();

	while(NRF_IRQ_PIN & (1<<NRF_IRQ));

	uint8_t status_reg;
	status_reg = NRF_ReadRegister(STATUS);

	if((status_reg & RX_DR) == RX_DR)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	if((status_reg & TX_DS) == TX_DS)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	if((status_reg & MAX_RT) == MAX_RT)
	{
		NRF_WriteRegister(0xff, STATUS);
		NRF_FlushTXBuffer();
	}
	NRF_CE_Lo;
	NRF_SetRXState();
	NRF_CE_Hi;
}
