/*********************************/
/******** Software SPI ***********/
/*********************************/

#include "SoftwareSPI_Dec.h"
#include <stdint.h>
#include <util/delay.h>

/******* SSPI Init **************/

void SSPI_Init(void)
{

	SMISO_DDR &= ~(1<<SMISO);					// Set SMISO as Input
	SMOSI_DDR |= (1<<SMOSI); 
	SSCK_DDR |= (1<<SSCK); 
	SCSN_DDR |= (1<<SCSN);						// Set SMOSI, SSCK and SCSN as Output

	SMISO_PORT |= (1<<SMISO); 					// Set Pullupresistor

	SSCK_Lo;
	SCSN_Hi;
}


/******* SSPI Write **************/

void SSPI_Write_byte(uint8_t Byte)
{
	uint8_t bits = 0x80;			// MSB first (LSB first => 0x01)
	
	while(bits)
	{
		if( Byte & bits )
		{
			Set_SMOSI;
		}
		else
		{
			Clear_SMOSI;
		}
		_delay_us(10);
		SSCK_Hi;
		_delay_us(10);
		SSCK_Lo;
		bits = bits>>1;				// schiebt nach links bis 1 rausfaellt (LSB fist <<)
	}
}

/******* SSPI Read **************/

uint8_t SSPI_Read_byte(uint8_t Byte)
{
	uint8_t bits = 0x80;			// MSB first (LSB first => 0x01)
	unsigned char data_receive = 0x00;
	
	while(bits)
	{
		if( Byte & bits )
		{
			Set_SMOSI;
		}
		else
		{
			Clear_SMOSI;
		}
		_delay_us(10);
		SSCK_Hi;
		_delay_us(10);
		if(SMISO_PIN & (1<<SMISO))
		{
			data_receive |= bits;
			
		}
		SSCK_Lo;
		bits = bits>>1;				// schiebt nach links bis 1 rausfaellt (LSB fist <<)
	}
	return data_receive;
}
