/**************************************/
/********** Switch Lights *************/
/**************************************/

#include "PWM_Dec_ATmega164P.h"
#include "SwitchLights_Dec.h"
#include "Converter.h"
#include "NRF24L01_Dec.h"
#include "NRF_Send_Receive_Dec.h"
#include <util/delay.h>

extern volatile long long int Switch1;
extern volatile long long int Switch2;
extern volatile long long int Switch3;
extern volatile long long int Switch4;
extern volatile unsigned char RelayState;
extern volatile unsigned char GateAddress[4];
extern volatile unsigned char RelayPWM[4][1];
extern volatile unsigned char RemoteAddress[40][4];

/**** Relay on/off Functions *****/

void Relay1_ON(uint8_t PWM)
{
	OC1B_PWM_Static();
	_delay_ms(10);
	OC1B_PWM_ON_8bit(PWM);
	RelayState |= 0x01;
}

void Relay1_OFF()
{
	OC1B_PWM_OFF();
	RelayState &= ~0x01;
}

void Relay2_ON(uint8_t PWM)
{
	OC1A_PWM_Static();
	_delay_ms(10);
	OC1A_PWM_ON_8bit(PWM);
	RelayState |= 0x02;
}

void Relay2_OFF()
{
	OC1A_PWM_OFF();
	RelayState &= ~0x02;
}

void Relay3_ON(uint8_t PWM)
{
	OC2B_PWM_Static();
	_delay_ms(10);
	OC2B_PWM_ON(PWM);
	RelayState |= 0x04;
}

void Relay3_OFF()
{
	OC2B_PWM_OFF();
	RelayState &= ~0x04;
}

void Relay4_ON(uint8_t PWM)
{
	OC2A_PWM_Static();
	_delay_ms(10);
	OC2A_PWM_ON(PWM);
	RelayState |= 0x08;
}

void Relay4_OFF()
{
	OC2A_PWM_OFF();
	RelayState &= ~0x08;
}

/**** Function for internal Groupswitching *****/

void SwitchInternalGroup(uint8_t SwitchNr)
{
	volatile int8_t On_Off_count = 0x00;
	volatile long long int tempSwitch;
	volatile uint8_t ExtSW = 0x00;

	Payloadconvert.c[0] = 0xFE;
	Payloadconvert.c[5] = 0x00;
	Payloadconvert.c[6] = 0x00;

	switch(SwitchNr)
	{	
		case 0x01: 
			tempSwitch = Switch1;
			ExtSW = 0x10;
			break;
		case 0x02:
			tempSwitch = Switch2;
			ExtSW = 0x20;
			break;
		case 0x03:
			tempSwitch = Switch3;
			ExtSW = 0x40;
			break;
		case 0x04:
			tempSwitch = Switch4;
			ExtSW = 0x80;
			break;
	}

	if((tempSwitch & 0x800000000000) == 0x800000000000)				// Switch controls Relay 4
	{
		if((RelayState & 0x01) == 0x01)				// Relay 4 is on
		{
			On_Off_count++;
		}
		else										// Relay 4 is off
		{
			On_Off_count--;
		}
	}
	if((tempSwitch & 0x400000000000) == 0x400000000000)				// Switch controls Relay 3
	{	
		if((RelayState & 0x02) == 0x02)				// Relay 3 is on
		{
			On_Off_count++;
		}
		else										// Relay 3 is off
		{
			On_Off_count--;
		}
	}
	if((tempSwitch & 0x200000000000) == 0x200000000000)				// Switch controls Relay 2
	{
		if((RelayState & 0x04) == 0x04)				// Relay 2 is on
		{
			On_Off_count++;
		}
		else										// Relay 2 is off
		{
			On_Off_count--;
		}
	}
	if((tempSwitch & 0x100000000000) == 0x100000000000)				// Switch controls Relay 1
	{
		if((RelayState & 0x08) == 0x08)				// Relay 1 is on
		{
			On_Off_count++;
		}
		else										// Relay 1 is off
		{
			On_Off_count--;
		}
	}
/*	if((tempSwitch & ~0xF00000000000) > 0x000000000000)				// Switch controls extern Relays 
	{
		if((RelayState & SwitchNr) == SwitchNr)		// extern group is on
		{
			On_Off_count++;
		}
		else										// extern group is off
		{
			On_Off_count--;
		}
	}*/
	if(On_Off_count <= 0)							// on/off difference is 0
	{
		if(tempSwitch & 0x800000000000)
		{
			Relay1_ON(RelayPWM[0][0]);

			Payloadconvert.c[7] = 0xFF;

			Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P1);

			for(int i=0; i<4; i++)
			{
				Payloadconvert.c[i+1] = Addressconvert.c[i];
			}

			SendToGate(Payloadconvert.u256i);
		}
		if(tempSwitch & 0x400000000000)
		{
			Relay2_ON(RelayPWM[1][0]);

			Payloadconvert.c[7] = 0xFF;

			Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P2);

			for(int i=0; i<4; i++)
			{
				Payloadconvert.c[i+1] = Addressconvert.c[i];
			}

			SendToGate(Payloadconvert.u256i);
		}
		if(tempSwitch & 0x200000000000)
		{
			Relay3_ON(RelayPWM[2][0]);

			Payloadconvert.c[7] = 0xFF;

			Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P3);

			for(int i=0; i<4; i++)
			{
				Payloadconvert.c[i+1] = Addressconvert.c[i];
			}

			SendToGate(Payloadconvert.u256i);
		}
		if(tempSwitch & 0x100000000000)
		{
			Relay4_ON(RelayPWM[3][0]);

			Payloadconvert.c[7] = 0xFF;

			Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P4);

			for(int i=0; i<4; i++)
			{
				Payloadconvert.c[i+1] = Addressconvert.c[i];
			}

			SendToGate(Payloadconvert.u256i);
		}
		if((tempSwitch & 0x0FFFFFFFFFFF) > 0x000000000000)
		{
			SwitchExternRelays(SwitchNr, tempSwitch, 0xFF);
			RelayState |= ExtSW;
		}
	}
	else
	{
		if(On_Off_count > 0)						// more lights are off
		{
			if(tempSwitch & 0x800000000000)
			{
				Relay1_OFF();

				Payloadconvert.c[7] = 0x00;

				Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P1);

				for(int i=0; i<4; i++)
				{
					Payloadconvert.c[i+1] = Addressconvert.c[i];
				}

				SendToGate(Payloadconvert.u256i);
			}
			if(tempSwitch & 0x400000000000)
			{
				Relay2_OFF();

				Payloadconvert.c[7] = 0x00;

				Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P2);

				for(int i=0; i<4; i++)
				{
					Payloadconvert.c[i+1] = Addressconvert.c[i];
				}

				SendToGate(Payloadconvert.u256i);
			}
			if(tempSwitch & 0x200000000000)
			{
				Relay3_OFF();

				Payloadconvert.c[7] = 0x00;

				Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P3);

				for(int i=0; i<4; i++)
				{
					Payloadconvert.c[i+1] = Addressconvert.c[i];
				}

				SendToGate(Payloadconvert.u256i);
			}
			if(tempSwitch & 0x100000000000)
			{
				Relay4_OFF();

				Payloadconvert.c[7] = 0x00;

				Addressconvert.u32i = NRF_ReadRXAddress(RX_ADDR_P4);

				for(int i=0; i<4; i++)
				{
					Payloadconvert.c[i+1] = Addressconvert.c[i];
				}

				SendToGate(Payloadconvert.u256i);
			}
			if((tempSwitch & 0x0FFFFFFFFFFF) > 0x000000000000)
			{
				SwitchExternRelays(SwitchNr, tempSwitch, 0x00);
				RelayState &= ~ExtSW;
			}
		}
	}
}

void SwitchExternRelays(uint8_t SwitchNr, long long int Switch, unsigned char Value)
{
	uint8_t i;
	
	Switch = (Switch<<4);

	for(i=0; i<40; i++)
	{
		if(((Switch<<i) & 0x800000000000) == 0x800000000000)
		{
			Addressconvert.c[0] = 0xE6; // RemoteAddress[i][0];
			Addressconvert.c[1] = 0xE6; // RemoteAddress[i][1];
			Addressconvert.c[2] = 0xE6; // RemoteAddress[i][2];
			Addressconvert.c[3] = 0xE6; // RemoteAddress[i][3];
			break;

			Payloadconvert.c[0] = 0xBF;
			Payloadconvert.c[1] = GateAddress[0];
			Payloadconvert.c[2] = GateAddress[1];
			Payloadconvert.c[3] = GateAddress[2];
			Payloadconvert.c[4] = GateAddress[3];
			Payloadconvert.c[5] = 0x00;
			Payloadconvert.c[6] = 0x00;
			Payloadconvert.c[7] = Value;

			SendtoLights(Payloadconvert.u256i, Addressconvert.u32i);
		}
	}
}
